@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11.2.2 Fri, 26 Apr 2002 18:24:52 +0400 jmacd $
+ * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11.2.3 Sun, 09 May 2004 18:21:12 -0700 jmacd $
  */
 
 
@@ -36,6 +36,8 @@ extern "C" {
 
 #include "misc.h"
 
+#include <ext/stdio_filebuf.h>
+
 static const int min_tty_width = 40;
 static char** argv_save = NULL;
 static int argc_save;
@@ -52,8 +54,8 @@ int return_if_fail_if_ne_val;
  stdiobuf stdout_stream(STDOUT_FILENO);
  stdiobuf stderr_stream(STDERR_FILENO);
 #else
- stdiobuf stdout_stream(stdout);
- stdiobuf stderr_stream(stderr);
+ __gnu_cxx::stdio_filebuf<char> stdout_stream(stdout, ios::out);
+ __gnu_cxx::stdio_filebuf<char> stderr_stream(stderr, ios::out);
 #endif /* if defined(__APPLE__) */
 strstreambuf query_stream;
 
@@ -101,17 +103,13 @@ int PrettyStreambuf::overflow(int c)
 {
     line_buffer.append(c);
     if(isspace(c))
-	xsputn(NULL, 0);
+	sputn(NULL, 0);
 
     return 1;
 }
 
 int PrettyStreambuf::xsputn(const char* s0, int n0)
 {
-#ifndef __GNUG__
-#  define xsputn sputn
-#endif
-
     if (dont_print && *dont_print)
 	return n0;
 
@@ -137,11 +135,11 @@ int PrettyStreambuf::xsputn(const char* s0, int n0)
 
 		if(!new_line) {
 		    for(int j = prefix.length(); j; j -= 1) {
-			forward->xsputn(" ", 1);
+			forward->sputn(" ", 1);
 			col += 1;
 		    }
 		} else {
-		    forward->xsputn(prefix.cast(), prefix.length());
+		    forward->sputn(prefix.cast(), prefix.length());
 		    col += prefix.length();
 		}
 
@@ -161,48 +159,40 @@ int PrettyStreambuf::xsputn(const char* s0, int n0)
 	    if(wordlen == 0) {
 		if(*s == '\n' || (col + 1 > width)) {
 		    new_line = false;
-		    forward->xsputn("\n", 1);
+		    forward->sputn("\n", 1);
 		    advance_buffer(s, n, col, 1);
 		    col = 0;
 		} else {
-		    forward->xsputn(s, 1);
+		    forward->sputn(s, 1);
 		    advance_buffer(s, n, col, 1);
 		}
 	    } else if(wordlen + col <= width) {
-		forward->xsputn(s, wordlen);
+		forward->sputn(s, wordlen);
 		advance_buffer(s, n, col, wordlen);
 	    } else if((wordlen + prefix.length() >= width) && (col == prefix.length())) {
-		forward->xsputn(s, wordlen);
+		forward->sputn(s, wordlen);
 		advance_buffer(s, n, col, wordlen);
 	    } else if(wordlen == 1 && col == width) {
 		new_line = false;
-		forward->xsputn(s, 1);
+		forward->sputn(s, 1);
 		advance_buffer(s, n, col, wordlen);
-		forward->xsputn("\n", 1);
+		forward->sputn("\n", 1);
 		col = 0;
 	    } else {
 		new_line = false;
-		forward->xsputn("\n", 1);
+		forward->sputn("\n", 1);
 		col = 0;
 	    }
 	}
     }
 
     return n;
-
-#ifndef __GNUG__
-#  undef xsputn
-#endif
 }
 
 int PrettyStreambuf::sync()
 {
-    xsputn(NULL, 0);
-#ifdef __GNUG__
-    return forward->sync();
-#else
+    sputn(NULL, 0);
     return forward->pubsync();
-#endif
 }
 
 static int tty_width(int fileno)
@@ -220,13 +210,8 @@ static int tty_width(int fileno)
 
 void re_query()
 {
-#ifdef __GNUG__
-    stdout_stream.xsputn(re_query_message, re_query_len);
-    stdout_stream.sync();
-#else
     stdout_stream.sputn(re_query_message, re_query_len);
     stdout_stream.pubsync();
-#endif
 }
 
 void continue_handler(SIGNAL_ARG_TYPE)
@@ -324,11 +309,7 @@ ostream& __omanip_setcol(ostream& o, int col)
 {
     PrettyStreambuf& ps = ((PrettyOstream&)o).ostreambuf();
 
-#ifdef __GNUG__
-    ps.xsputn(NULL, 0);
-#else
     ps.sputn(NULL, 0);
-#endif
     ps.set_column(col);
 
     return o;
@@ -389,7 +370,7 @@ int PrettyStreambuf::set_column(int col0)
 
     Dstring spaces(' ', diff);
     bool cb = set_fill_break(false);
-    xsputn(spaces.cast(), diff);
+    sputn(spaces.cast(), diff);
     set_fill_break(cb);
 
     return col0;
@@ -405,14 +386,12 @@ ostream& prcsendl(ostream& s)
 {
     PrettyStreambuf &ps = ((PrettyOstream&)s).ostreambuf();
 
-#ifdef __GNUG__
-    ps.xsputn("\n", 1);
-#else
     ps.sputn("\n", 1);
-#endif
 
     ps.reset_column();
 
+    ps.pubsync ();
+
     return s;
 }
 
@@ -420,14 +399,12 @@ ostream& dotendl(ostream& s)
 {
     PrettyStreambuf &ps = ((PrettyOstream&)s).ostreambuf();
 
-#ifdef __GNUG__
-    ps.xsputn(".\n", 2);
-#else
     ps.sputn(".\n", 2);
-#endif
 
     ps.reset_column();
 
+    ps.pubsync ();
+
     return s;
 }
 
@@ -439,6 +416,8 @@ ostream& perror(ostream& s)
 
     ps.reset_column();
 
+    ps.pubsync ();
+
     return s;
 }
 
@@ -454,12 +433,9 @@ PrettyStreambuf::PrettyStreambuf(streambuf *forward0, int* dont_print0)
 /* QueryOstream */
 QueryOstream::QueryOstream(strstreambuf* base_stream0,
 			   PrettyStreambuf* query_stream0,
-			   stdiobuf* stdout_stream0,
-			   stdiobuf* stderr_stream0)
-    :
-#ifndef __GNUG__
-     ios(query_stream0),
-#endif
+			   filebuf         *stdout_stream0,
+			   filebuf         *stderr_stream0)
+    :ios(query_stream0),
      PrettyOstream(query_stream0, NoError),
      base_stream(base_stream0),
      query_stream(query_stream0),
@@ -516,35 +492,30 @@ ostream& QueryOstream::bang(BangFlag* flag)
 
 ostream& QueryOstream::string_query_manip(const char* message)
 {
-#ifndef __GNUG__
-#  define xsputn       sputn
-#  define sync         pubsync
-#  define seekoff(p,w) pubseekoff((p),(w), ios::out)
-#endif
     if(option_force_resolution && option_be_silent) {
 	/* silence */
     } else if(option_force_resolution) {
 	*this << " -- " << default_input << prcsendl;
 	string_val = PrConstCharPtrError(default_input);
-	stderr_stream->xsputn(base_stream->str(),
+	stderr_stream->sputn(base_stream->str(),
 			      base_stream->out_waiting());
-	stderr_stream->sync();
+	stderr_stream->pubsync();
     } else if(option_report_actions) {
 	*this << " -- " << default_input << prcsendl;
 	string_val = PrConstCharPtrError(default_input);
-	stdout_stream->xsputn(base_stream->str(),
+	stdout_stream->sputn(base_stream->str(),
 			      base_stream->out_waiting());
-	stdout_stream->sync();
+	stdout_stream->pubsync();
     } else {
 	*this << "[" << default_input << "] " << message;
-	query_stream->sync();
+	query_stream->pubsync();
 
 	re_query_message = base_stream->str();
 	re_query_len = base_stream->out_waiting();
 
 	while(true) {
-	    stdout_stream->xsputn(re_query_message, re_query_len);
-	    stdout_stream->sync();
+	    stdout_stream->sputn(re_query_message, re_query_len);
+	    stdout_stream->pubsync();
 
 	    Dstring *ans = new Dstring; /* leak */
 
@@ -570,18 +541,13 @@ ostream& QueryOstream::string_query_manip(const char* message)
 
     query_stream->reset_column();
     base_stream->freeze(0);
-    base_stream->seekoff(0, ios::beg);
+    base_stream->pubseekoff(0, ios::beg, ios::out);
     option_count = 0;
     force_option = 0;
     bang_flag = NULL;
     help_string = NULL;
 
     return *this;
-#ifndef __GNUG__
-#  undef xsputn
-#  undef sync
-#  undef seekoff
-#endif
 }
 
 ostream& QueryOstream::help_manip(const char* message)
@@ -592,11 +558,6 @@ ostream& QueryOstream::help_manip(const char* message)
 
 ostream& QueryOstream::query_manip(const char* message)
 {
-#ifndef __GNUG__
-#  define xsputn  sputn
-#  define sync    pubsync
-#  define seekoff(p,w) pubseekoff((p),(w), ios::out)
-#endif
     if(force_option != 0 && option_force_resolution) {
         for (int i = 0; i < option_count; i += 1) {
 	    if (force_option == options[i].let) {
@@ -611,24 +572,24 @@ ostream& QueryOstream::query_manip(const char* message)
 	val = options[default_option].val;
     } else if(option_report_actions) {
 	*this << report_message << dotendl;
-	query_stream->sync();
-	stdout_stream->xsputn(base_stream->str(),
+	query_stream->pubsync();
+	stdout_stream->sputn(base_stream->str(),
 			      base_stream->out_waiting());
-	stdout_stream->sync();
+	stdout_stream->pubsync();
 	val = options[default_option].val;
     } else if(option_force_resolution) {
 	*this << force_message << dotendl;
-	query_stream->sync();
-	stderr_stream->xsputn(base_stream->str(),
+	query_stream->pubsync();
+	stderr_stream->sputn(base_stream->str(),
 			      base_stream->out_waiting());
-	stderr_stream->sync();
+	stderr_stream->pubsync();
 	val = options[default_option].val;
     } else if(bang_flag && bang_flag->flag) {
 	*this << force_message << dotendl;
-	query_stream->sync();
-	stdout_stream->xsputn(base_stream->str(),
+	query_stream->pubsync();
+	stdout_stream->sputn(base_stream->str(),
 			      base_stream->out_waiting());
-	stdout_stream->sync();
+	stdout_stream->pubsync();
 	val = options[default_option].val;
     } else {
 	char query_buf[40];
@@ -657,7 +618,7 @@ ostream& QueryOstream::query_manip(const char* message)
 
 	*this << message << query_buf;
 
-	query_stream->sync();
+	query_stream->pubsync();
 
 	re_query_message = base_stream->str();
 	re_query_len = base_stream->out_waiting();
@@ -666,8 +627,8 @@ ostream& QueryOstream::query_manip(const char* message)
 	    char c;
 	    int found = false;
 
-	    stdout_stream->xsputn(re_query_message, re_query_len);
-	    stdout_stream->sync();
+	    stdout_stream->sputn(re_query_message, re_query_len);
+	    stdout_stream->pubsync();
 
 	    c = get_user_char();
 
@@ -718,18 +679,13 @@ ostream& QueryOstream::query_manip(const char* message)
 
     query_stream->reset_column();
     base_stream->freeze(0);
-    base_stream->seekoff(0, ios::beg);
+    base_stream->pubseekoff(0, ios::beg, ios::out);
     force_option = 0;
     option_count = 0;
     bang_flag = NULL;
     help_string = NULL;
 
     return *this;
-#ifndef __GNUG__
-#  undef xsputn
-#  undef sync
-#  undef seekoff
-#endif
 }
 
 ostream& __omanip_query(ostream& s, const char* message) {
