@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.14.1.5 Fri, 26 Apr 2002 17:41:07 +0400 jmacd $
+ * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.14.1.6 Sun, 09 May 2004 18:21:12 -0700 jmacd $
  */
 
 extern "C" {
@@ -40,6 +40,10 @@ extern "C" {
 #include "syscmd.h"
 #include "checkin.h"
 
+#if __GNUG__ > 3 || (__GNUG__ == 3 && __GNUC_MINOR__ >= 2)
+#include <ext/stdio_filebuf.h>
+#endif
+
 struct RebuildCallbackData {
     RepEntry* rep_entry;
     RcsFileTable* file_table;
@@ -994,9 +998,13 @@ void RebuildFile::init_stream()
 #if __GNUG__ < 3
 	buf = new filebuf(seg->fd());
 	buf->seekoff(0,ios::end);
-#else
+#elif __GNUG__ == 3 and __GNUC_MINOR__ < 2
         buf = new filebuf(fdopen(dup(seg->fd()), "a+"), ios::out);
         buf->pubseekoff(0, ios::end, ios::out);
+#else
+        buf = new __gnu_cxx::stdio_filebuf<char> (seg->fd(), ios::out,
+						  false /* close */, default_segment_size);
+        buf->pubseekoff(0, ios::end, ios::out);
 #endif
 	os = new ostream(buf);
     }
